@@ -1,7 +1,7 @@
 #' Create a list environment
 #'
-#' @param \dots (optional) Named and/or unnamed objects to be
-#' assigned to the list environment.
+#' @param \dots (optional) Named and/or unnamed objects to be assigned to the
+#' list environment.
 #'
 #' @return An environment of class `listenv`.
 #'
@@ -15,31 +15,23 @@ listenv <- function(...) {
   names <- names(args)
 
   ## Allocate empty list environment
-  metaenv <- new.env(parent=parent.frame())
-  env <- new.env(parent=metaenv)
+  metaenv <- new.env(parent = parent.frame())
+  env <- new.env(parent = metaenv)
 
-  ## Add elements?
-  if (nargs > 0L) {
-    ## Backward compatibility
-    if (nargs == 1L && identical(names[1L], "length")) {
-      .Deprecated(msg="Use of x <- listenv(length=n) to allocate a list environment of length n is deprecated. Use x <- listenv(); length(x) <- n instead.")
-      length <- args$length
-      stopifnot(length >= 0L)
-      args <- vector("list", length=length)
-      nargs <- length
-      names <- NULL
-    }
+  ## Defunct call?
+  if (nargs == 1L && identical(names[1L], "length")) {
+    .Defunct(msg = "Use of x <- listenv(length = n) to allocate a list environment of length n is defunct. Use x <- listenv(); length(x) <- n instead.")  #nolint
   }
 
   ## Allocate internal variables
   maps <- sprintf(".listenv_var_%d", seq_len(nargs))
   names(maps) <- names
   for (kk in seq_len(nargs)) {
-    assign(maps[kk], value=args[[kk]], envir=env, inherits=FALSE)
+    assign(maps[kk], value = args[[kk]], envir = env, inherits = FALSE)
   }
-  metaenv$.listenv.map <- maps
+  metaenv[[".listenv.map"]] <- maps
 
-  assign(".listenv_var_count", nargs, envir=env, inherits=FALSE)
+  assign(".listenv_var_count", nargs, envir = env, inherits = FALSE)
 
   class(env) <- c("listenv", class(env))
 
@@ -107,21 +99,24 @@ print.listenv <- function(x, ...) {
   }
 
   s <- sprintf("A %s %s with %d", sQuote(class), what, n)
-  if (is.null(names) && n > 0) {
-    s <- sprintf("%s unnamed", s)
-  }
   if (n == 1) {
     s <- sprintf("%s element", s)
   } else {
     s <- sprintf("%s elements", s)
   }
-  if (!is.null(names)) {
-    s <- sprintf("%s (%s)", s, hpaste(sQuote(names)))
+  if (is.null(names)) {
+    s <- sprintf("%s (unnamed)", s)
+  } else {
+    if (n == 0) {
+      s <- sprintf("%s (named)", s)
+    } else {
+      s <- sprintf("%s (%s)", s, hpaste(sQuote(names)))
+    }
   }
   if (ndim > 1) {
-    dimstr <- paste(dim, collapse="x")
-    hasDimnames <- !sapply(dimnames, FUN=is.null)
-    dimnamesT <- sapply(dimnames, FUN=function(x) hpaste(sQuote(x)))
+    dimstr <- paste(dim, collapse = "x")
+    has_dimnames <- !sapply(dimnames, FUN = is.null)
+    dimnames_tmp <- sapply(dimnames, FUN = function(x) hpaste(sQuote(x)))
 
     s <- sprintf("%s arranged in %s", s, dimstr)
 
@@ -129,12 +124,13 @@ print.listenv <- function(x, ...) {
       if (is.null(dimnames)) {
         s <- sprintf("%s unnamed rows and columns", s, dimstr)
       } else {
-        if (all(hasDimnames)) {
-          s <- sprintf("%s rows (%s) and columns (%s)", s, dimnamesT[1L], dimnamesT[2L])
-        } else if (hasDimnames[1]) {
-          s <- sprintf("%s rows (%s) and unnamed columns", s, dimnamesT[1L])
-        } else if (hasDimnames[2]) {
-          s <- sprintf("%s unnamed rows and columns (%s)", s, dimnamesT[2L])
+        if (all(has_dimnames)) {
+          s <- sprintf("%s rows (%s) and columns (%s)", s,
+                       dimnames_tmp[1L], dimnames_tmp[2L])
+        } else if (has_dimnames[1]) {
+          s <- sprintf("%s rows (%s) and unnamed columns", s, dimnames_tmp[1L])
+        } else if (has_dimnames[2]) {
+          s <- sprintf("%s unnamed rows and columns (%s)", s, dimnames_tmp[2L])
         } else {
           s <- sprintf("%s unnamed rows and columns", s, dimstr)
         }
@@ -143,15 +139,16 @@ print.listenv <- function(x, ...) {
       if (is.null(dimnames)) {
         s <- sprintf("%s unnamed dimensions", s)
       } else {
-        dimnamesT[!hasDimnames] <- "NULL"
-        dimnamesT <- sprintf("#%d: %s", seq_along(dimnamesT), dimnamesT)
-        dimnamesT <- paste(dimnamesT, collapse="; ")
-        if (all(hasDimnames)) {
-          s <- sprintf("%s dimensions (%s)", s, dimnamesT)
-        } else if (!any(hasDimnames)) {
+        dimnames_tmp[!has_dimnames] <- "NULL"
+        dimnames_tmp <- sprintf("#%d: %s",
+                                seq_along(dimnames_tmp), dimnames_tmp)
+        dimnames_tmp <- paste(dimnames_tmp, collapse = "; ")
+        if (all(has_dimnames)) {
+          s <- sprintf("%s dimensions (%s)", s, dimnames_tmp)
+        } else if (!any(has_dimnames)) {
           s <- sprintf("%s unnamed dimensions", s)
         } else {
-          s <- sprintf("%s partially named dimensions (%s)", s, dimnamesT)
+          s <- sprintf("%s partially named dimensions (%s)", s, dimnames_tmp)
         }
       }
     }
@@ -165,18 +162,29 @@ print.listenv <- function(x, ...) {
 #'
 #' @param x A list environment.
 #'
-#' @return The a named character vector
+#' @return A named character vector
 #'
+#' @details
+#' _Functions `map()` and `map<-()` have been renamed to `mapping()` and
+#' `mapping<-()`. The former will soon become deprecated and eventually
+#' defunct. Please update accordingly._
+#' 
+#' @aliases mapping.listenv
 #' @aliases map.listenv
 #' @export
 #' @keywords internal
-map <- function(x, ...) {
-  get(".listenv.map", envir=parent.env(x), inherits=FALSE)
+mapping <- function(x, ...) {
+  get(".listenv.map", envir = parent.env(x), inherits = FALSE)
 }
 
-`map<-` <- function(x, value) {
-  stopifnot(is.character(value))
-  assign(".listenv.map", value, envir=parent.env(x), inherits=FALSE)
+#' @rdname mapping
+#' @export
+#' @keywords internal
+map <- mapping
+
+`mapping<-` <- function(x, value) {
+  stop_if_not(is.character(value))
+  assign(".listenv.map", value, envir = parent.env(x), inherits = FALSE)
   invisible(x)
 }
 
@@ -184,19 +192,31 @@ map <- function(x, ...) {
 #'
 #' @param x A list environment.
 #'
+#' @aliases lengths.listenv
 #' @export
 #' @keywords internal
 length.listenv <- function(x) {
-  length(map(x))
+  length(mapping(x))
+}
+
+## BACKPORT / WORKAROUND:
+## lengths() was introduced in R 3.2.0, but only became a generic in R 3.3.0.
+## Since this packages is supported on R (>= 3.1.2), declaring above methods
+## as S3method() in the NAMESPACE would given an error on R (< 3.2.0).
+## Because of this, lengths() is declared as a generic if missing, i.e.
+## in R (< 3.2.0).  This will make lengths() for list environments to work
+## with R (<= 3.2.0) and R (>= 3.3.0) but not with R 3.2.x versions.
+if (!exists("lengths", mode = "function")) {
+  lengths <- function(x, use.names = FALSE) UseMethod("lengths")  #nolint
 }
 
 #' @export
 `length<-.listenv` <- function(x, value) {
-  map <- map(x)
+  map <- mapping(x)
   n <- length(map)
   value <- as.numeric(value)
 
-  if (value < 0) stop("invalid value")
+  if (value < 0) stop("Cannot set a negative length")
 
   ## Nothing to do?
   if (value == n) return(invisible(x))
@@ -204,18 +224,18 @@ length.listenv <- function(x) {
   ## Expand or shrink?
   if (value > n) {
     ## Add place holders for added elements
-    extra <- rep(NA_character_, times=value-n)
+    extra <- rep(NA_character_, times = value - n)
     map <- c(map, extra)
   } else {
     ## Drop existing variables
-    drop <- (value+1):n
+    drop <- (value + 1):n
     var <- map[drop]
     ## Some may be internal place holders
     var <- var[!is.na(var)]
-    if (length(var) > 0) remove(list=var, envir=x, inherits=FALSE)
+    if (length(var) > 0) remove(list = var, envir = x, inherits = FALSE)
     map <- map[-drop]
   }
-  map(x) <- map
+  mapping(x) <- map
 
   invisible(x)
 }
@@ -229,39 +249,47 @@ length.listenv <- function(x) {
 #' @export
 #' @keywords internal
 names.listenv <- function(x) {
-  names(map(x))
+  names(mapping(x))
 }
 
 #' @export
 `names<-.listenv` <- function(x, value) {
-  map <- map(x)
+  map <- mapping(x)
   if (is.null(value)) {
   } else if (length(value) != length(map)) {
-    stop(sprintf("Number of names does not match the number of elements: %s != %s", length(value), length(map)))
+    stopf("The number of names does not match the number of elements: %s != %s",
+          length(value), length(map))
   }
-##  if (any(duplicated(value))) {
-##    stop("Environments cannot have duplicate names on elements")
-##  }
   names(map) <- value
-  map(x) <- map
+  mapping(x) <- map
   invisible(x)
 }
 
+#' @exportS3Method lengths listenv
+lengths.listenv <- function(x, use.names = TRUE) {  #nolint
+  ns <- lapply(x, FUN = length)
+  if (length(ns) == 0L) return(integer(0L))
+  unlist(ns, use.names = use.names)
+}
+
 #' List representation of a list environment
 #'
 #' @param x A list environment.
-#' @param all.names If \code{TRUE}, variable names starting with
-#'        a period are included, otherwise not.
-#' @param sorted If \code{TRUE}, elements are ordered by their names
-#'        before being compared, otherwise not.
+#' 
+#' @param all.names If `TRUE`, variable names starting with a period are
+#' included, otherwise not.
+#' 
+#' @param sorted If `TRUE`, elements are ordered by their names before being
+#' compared, otherwise not.
+#' 
 #' @param ... Not used.
 #'
 #' @return A list.
 #'
 #' @export
 #' @keywords internal
-as.list.listenv <- function(x, all.names=TRUE, sorted=FALSE, ...) {
-  vars <- map(x)
+as.list.listenv <- function(x, all.names = TRUE, sorted = FALSE, ...) {
+  vars <- mapping(x)
   nvars <- length(vars)
   names <- names(x)
 
@@ -281,12 +309,12 @@ as.list.listenv <- function(x, all.names=TRUE, sorted=FALSE, ...) {
   }
 
   ## Collect as a named list
-  res <- vector("list", length=nvars)
+  res <- vector("list", length = nvars)
   names(res) <- names
 
   if (nvars > 0) {
     ok <- !is.na(vars)
-    res[ok] <- mget(vars[ok], envir=x, inherits=FALSE)
+    res[ok] <- mget(vars[ok], envir = x, inherits = FALSE)
   }
 
   ## Set dimensions?
@@ -304,9 +332,10 @@ as.list.listenv <- function(x, all.names=TRUE, sorted=FALSE, ...) {
 #' Get elements of list environment
 #'
 #' @param x A list environment.
+#' 
 #' @param name The name or index of the element to retrieve.
 #'
-#' @return The value of an element or NULL if the element does not exist
+#' @return The value of an element or `NULL` if the element does not exist.
 #'
 #' @aliases [[.listenv
 #' @aliases [.listenv
@@ -314,28 +343,28 @@ as.list.listenv <- function(x, all.names=TRUE, sorted=FALSE, ...) {
 #' @keywords internal
 `$.listenv` <- function(x, name) {
 #' @keywords internal
-  map <- map(x)
+  map <- mapping(x)
   var <- map[name]
 
-  ## Non-existing variable?
+  # Non-existing variable?
   if (is.na(var)) return(NULL)
 
-  get(var, envir=x, inherits=FALSE)
+  get(var, envir = x, inherits = FALSE)
 }
 
 
 ## [[i,j,...]] -> [[idx]]
-toIndex <- function(x, idxs) {
+to_index <- function(x, idxs) {
   nidxs <- length(idxs)
 
   dim <- dim(x)
   if (is.null(dim)) dim <- length(x)
   ndim <- length(dim)
-  if (ndim != nidxs) {
-    stop("incorrect number of dimensions")
+  if (nidxs != ndim) {
+    stopf("Incorrect number of dimensions: %d != %d", nidxs, ndim)
   }
   dimnames <- dimnames(x)
-  idxDimnames <- dimnames
+  idx_dimnames <- dimnames
 
   ## Indexing scale factor per dimension
   scale <- c(1L, cumprod(dim[-ndim]))
@@ -347,20 +376,30 @@ toIndex <- function(x, idxs) {
     ni <- length(i)
     if (is.character(i)) {
       name <- i
-      i <- match(name, table=dimnames[[kk]])
-      if (anyNA(i)) stop("subscript out of bounds")
+      i <- match(name, table = dimnames[[kk]])
+      if (anyNA(i)) {
+        unknown <- name[is.na(i)]
+        stopf("Unknown names for dimension #%d: %s",
+	      kk, hpaste(sQuote(unknown)))
+      }
     } else if (is.logical(i)) {
       d <- dim[kk]
       ni <- length(i)
-      if (ni > d) stop("(subscript) logical subscript too long")
-      if (ni < d) i <- rep(i, length.out=d)
+      if (ni > d) {
+        stopf("Logical subscript for dimension #%d too long: %d > %d",
+	      kk, ni, d)
+      }
+      if (ni < d) i <- rep(i, length.out = d)
       i <- which(i)
     } else if (is.numeric(i)) {
       d <- dim[kk]
-      if (any(i > d)) stop("subscript out of bounds")
+      if (any(i > d)) {
+        stopf("Subscript for dimension #%d out of bounds [%d,%d]",
+	      kk, min(1, d), d)
+      }
       if (any(i < 0)) {
         if (any(i > 0)) {
-          stop("only 0's may be mixed with negative subscripts")
+          stopf("Only 0's may be mixed with negative subscripts (dimension #%d)", kk)
         }
         ## Drop elements
         i <- setdiff(seq_len(d), -i)
@@ -368,20 +407,21 @@ toIndex <- function(x, idxs) {
       ## Drop zeros
       i <- i[i != 0]
     } else {
-      stop("invalid subscript type", sQuote(typeof(i)))
+      stopf("Invalid subscript type for dimension #%d: %s",
+            kk, sQuote(typeof(i)))
     }
 
     ## Subset dimnames?
-    if (!is.null(idxDimnames)) {
-      dn <- idxDimnames[[kk]]
-      if (!is.null(dn)) idxDimnames[[kk]] <- dn[i]
+    if (!is.null(idx_dimnames)) {
+      dn <- idx_dimnames[[kk]]
+      if (!is.null(dn)) idx_dimnames[[kk]] <- dn[i]
     }
 
-    i <- scale[kk]*(i - 1)
+    i <- scale[kk] * (i - 1)
     if (kk == 1) {
       idx <- idx + i
     } else {
-      idx <- outer(idx, i, FUN=`+`)
+      idx <- outer(idx, i, FUN = `+`)
     }
   } # for (kk ...)
 
@@ -389,23 +429,22 @@ toIndex <- function(x, idxs) {
   dim <- dim(idx)
   ndim <- length(dim)
   if (ndim != nidxs) {
-    stop(sprintf("INTERNAL ERROR: Incompatible dimensions: %d != %d", ndim, nidxs))
+    stopf("INTERNAL ERROR: Incompatible dimensions: %d != %d", ndim, nidxs)
   }
 
   ## Preserve names(dim)
   names(dim(idx)) <- names(dim(x))
 
   ## Preserve dimnames
-  dimnames(idx) <- idxDimnames
-
+  dimnames(idx) <- idx_dimnames
 
   idx
-} # toIndex()
+}
 
 
 #' @export
 `[[.listenv` <- function(x, ...) {
-  map <- map(x)
+  map <- mapping(x)
   n <- length(map)
 
   idxs <- list(...)
@@ -413,39 +452,40 @@ toIndex <- function(x, idxs) {
 
   ## Subsetting by multiple dimensions?
   if (nidxs > 1L) {
-    i <- toIndex(x, idxs)
+    i <- to_index(x, idxs)
   } else {
     i <- idxs[[1L]]
     if (is.character(i)) {
       name <- i
-      i <- match(name, table=names(map))
+      i <- match(name, table = names(map))
       if (is.na(i)) return(NULL)
     } else if (!is.numeric(i)) {
-      return(NextMethod("[["))
+      return(NextMethod())
     }
 
     if (length(i) != 1L) {
-      stop("Subsetting of more than one element at the time is not allowed for listenv's: ", length(i))
+      stop("Subsetting of more than one element at the time is not allowed for listenv's: ", length(i))  #nolint
     }
 
     if (i < 1L || i > n) {
-      stop(sprintf("Subscript out of bounds [%d,%d]: %d", min(1,n), n, i), call.=FALSE)
+      stopf("Subscript out of bounds [%d,%d]: %d",
+            min(1, n), n, i, call. = FALSE)
     }
   }
 
   var <- map[i]
 
   ## Return default (NULL)?
-  if (is.na(var) || !exists(var, envir=x, inherits=FALSE)) return(NULL)
+  if (is.na(var) || !exists(var, envir = x, inherits = FALSE)) return(NULL)
 
-  get(var, envir=x, inherits=FALSE)
+  get(var, envir = x, inherits = FALSE)
 }
 
 
 #' @export
-`[.listenv` <- function(x, ..., drop=TRUE) {
-  ## Need to allow for implicit indices, e.g. x[1,,2]
-  idxs <- as.list(sys.call())[-(1:2)]
+`[.listenv` <- function(x, ..., drop = TRUE) {
+  # Need to allow for implicit indices, e.g. x[1,,2]
+  idxs <- as.list(sys.call())[-(1:2)]  #nolint
   idxs$drop <- NULL
   nidxs <- length(idxs)
 
@@ -453,11 +493,14 @@ toIndex <- function(x, idxs) {
   dim <- dim(x)
   ndim <- length(dim)
   if (nidxs > 1 && nidxs != ndim) {
-    stop(sprintf("Incorrect subsetting. Expected %d dimensions but got %d", ndim, nidxs))
+    stopf("Incorrect subsetting. Expected %d dimensions but got %d",
+          ndim, nidxs)
   }
 
   ## Implicitly specified dimensions
-  missing <- sapply(idxs, FUN=function(x) is.symbol(x) && identical("", deparse(x)))
+  missing <- sapply(idxs, FUN = function(x) {
+    is.symbol(x) && identical("", deparse(x))
+  })
   if (any(missing)) {
     if (nidxs == ndim) {
       envir <- parent.frame()
@@ -465,30 +508,30 @@ toIndex <- function(x, idxs) {
         if (missing[kk]) {
           idxs[[kk]] <- seq_len(dim[kk])
         } else {
-          idxs[[kk]] <- eval(idxs[[kk]], envir=envir)
+          idxs[[kk]] <- eval(idxs[[kk]], envir = envir, enclos = baseenv())
         }
       }
     } else if (nidxs == 1) {
       if (ndim == 0) {
         idxs <- list(seq_len(length(x)))
       } else {
-        ## Special case: Preserve dimensions when x[]
-        idxs <- lapply(dim, FUN=function(n) seq_len(n))
+        # Special case: Preserve dimensions when x[]
+        idxs <- lapply(dim, FUN = function(n) seq_len(n))
         nidxs <- length(idxs)
      }
     }
   } else {
     envir <- parent.frame()
-    idxs <- lapply(idxs, FUN=eval, envir=envir)
+    idxs <- lapply(idxs, FUN = eval, envir = envir, enclos = baseenv())
   }
 
   if (nidxs <= 1L) {
     i <- idxs[[1L]]
   } else {
-    i <- toIndex(x, idxs)
+    i <- to_index(x, idxs)
   }
 
-  map <- map(x)
+  map <- mapping(x)
   nmap <- length(map)
   names <- names(map)
 
@@ -496,13 +539,13 @@ toIndex <- function(x, idxs) {
     i <- integer(0L)
   } else if (is.character(i)) {
     name <- i
-    i <- match(name, table=names)
+    i <- match(name, table = names)
   } else if (is.numeric(i)) {
     ## Exclude elements with negative indices?
     if (any(i < 0)) {
-      stopifnot(is.null(dim(i)))
+      stop_if_not(is.null(dim(i)))
       if (any(i > 0)) {
-        stop("only 0's may be mixed with negative subscripts")
+        stop("Only 0's may be mixed with negative subscripts")
       }
       ## Drop elements
       i <- setdiff(seq_len(nmap), -i)
@@ -512,10 +555,10 @@ toIndex <- function(x, idxs) {
       i <- i[i != 0]
     }
   } else if (is.logical(i)) {
-    if (length(i) < nmap) i <- rep(i, length.out=nmap)
+    if (length(i) < nmap) i <- rep(i, length.out = nmap)
     i <- which(i)
   } else {
-    return(NextMethod("["))
+    return(NextMethod())
   }
 
   ## Nothing to do?
@@ -524,17 +567,17 @@ toIndex <- function(x, idxs) {
   ## Allocate result
   res <- listenv()
   length(res) <- ni
-  res <- structure(res, class=class(x))
+  res <- structure(res, class = class(x))
 
   if (ni > 0L) {
-    ## Add names?
+    # Add names?
     if (!is.null(names)) {
       names2 <- names[i]
       names2[i > nmap] <- ""
       names(res) <- names2
     }
 
-    ## Ignore out-of-range indices
+    # Ignore out-of-range indices
     j <- i[i <= nmap]
     for (kk in seq_along(j)) {
       value <- x[[j[kk]]]
@@ -568,35 +611,36 @@ toIndex <- function(x, idxs) {
 }
 
 
-new_variable <- function(envir, value, create=TRUE) {
-  count <- get(".listenv_var_count", envir=envir, inherits=FALSE)
+new_variable <- function(envir, value, create = TRUE) {
+  count <- get(".listenv_var_count", envir = envir, inherits = FALSE)
 
   count <- count + 1L
   name <- sprintf(".listenv_var_%f", count)
 
   if (!missing(value)) {
-    assign(name, value, envir=envir, inherits=FALSE)
+    assign(name, value, envir = envir, inherits = FALSE)
   }
 
   if (create) {
-    assign(".listenv_var_count", count, envir=envir, inherits=FALSE)
+    assign(".listenv_var_count", count, envir = envir, inherits = FALSE)
   }
 
   name
-} # new_variable()
+}
 
 
 assign_by_name <- function(x, name, value) {
   ## Argument 'name':
   if (length(name) == 0L) {
-    stop("Cannot assign value. Zero-length name.", call.=FALSE)
+    stop("Cannot assign value. Zero-length name.", call. = FALSE)
   } else if (length(name) > 1L) {
-    stop("Cannot assign value. More than one name specified: ", hpaste(name), call.=FALSE)
+    stop("Cannot assign value. More than one name specified: ",
+         hpaste(sQuote(name)), call. = FALSE)
   } else if (nchar(name) == 0L) {
-    stop("Cannot assign value. Empty name specific: ", name, call.=FALSE)
+    stop("Cannot assign value. Empty name specific: ", sQuote(name), call. = FALSE)
   }
 
-  map <- map(x)
+  map <- mapping(x)
   names <- names(map)
 
   ## Map to an existing or a new element?
@@ -607,39 +651,40 @@ assign_by_name <- function(x, name, value) {
     if (is.na(var)) {
       var <- name
       map[name] <- name
-      map(x) <- map
+      mapping(x) <- map
     }
   } else {
     var <- name
 
     ## Append to map
     map <- c(map, var)
-    if (is.null(names)) names <- rep("", times=length(map))
+    if (is.null(names)) names <- rep("", times = length(map))
     names[length(map)] <- var
     names(map) <- names
-    map(x) <- map
+    mapping(x) <- map
   }
 
   ## Assign value
-  assign(var, value, envir=x, inherits=FALSE)
+  assign(var, value, envir = x, inherits = FALSE)
 
   invisible(x)
-} # assign_by_name()
+}
 
 
 assign_by_index <- function(x, i, value) {
   ## Argument 'i':
   if (length(i) == 0L) {
-    stop("Cannot assign value. Zero-length index.", call.=FALSE)
+    stop("Cannot assign value. Zero-length index.", call. = FALSE)
   } else if (length(i) > 1L) {
-    stop("Cannot assign value. More than one index specified: ", hpaste(i), call.=FALSE)
+    stop("Cannot assign value. More than one index specified: ", hpaste(i),
+         call. = FALSE)
   } else if (!is.finite(i)) {
-    stop("Cannot assign value. Non-finite index: ", i, call.=FALSE)
+    stop("Cannot assign value. Non-finite index: ", i, call. = FALSE)
   } else if (i < 1L) {
-    stop("Cannot assign value. Non-positive index: ", i, call.=FALSE)
+    stop("Cannot assign value. Non-positive index: ", i, call. = FALSE)
   }
 
-  map <- map(x)
+  map <- mapping(x)
   n <- length(map)
 
   ## Variable name
@@ -649,34 +694,36 @@ assign_by_index <- function(x, i, value) {
   if (is.na(var)) {
     ## Expand map?
     if (i > n) {
-      extra <- rep(NA_character_, times=i-n)
+      extra <- rep(NA_character_, times = i - n)
       map <- c(map, extra)
     }
 
     ## Create internal variable
-    map[i] <- new_variable(x, value=value)
+    map[i] <- new_variable(x, value = value)
 
     ## Update map
-    map(x) <- map
+    mapping(x) <- map
   } else {
-    assign(var, value, envir=x, inherits=FALSE)
+    assign(var, value, envir = x, inherits = FALSE)
   }
 
   invisible(x)
-} # assign_by_index()
+}
 
 
 remove_by_name <- function(x, name) {
   ## Argument 'name':
   if (length(name) == 0L) {
-    stop("Cannot remove element. Zero-length name.", call.=FALSE)
+    stop("Cannot remove element. Zero-length name.", call. = FALSE)
   } else if (length(name) > 1L) {
-    stop("Cannot remove element. More than one name specified: ", hpaste(name), call.=FALSE)
+    stop("Cannot remove element. More than one name specified: ",
+         hpaste(sQuote(name)), call. = FALSE)
   } else if (nchar(name) == 0L) {
-    stop("Cannot remove element. Empty name specific: ", name, call.=FALSE)
+    stop("Cannot remove element. Empty name specific: ",
+         sQuote(name), call. = FALSE)
   }
 
-  map <- map(x)
+  map <- mapping(x)
 
   ## Position in names map?
   idx <- match(name, names(map))
@@ -686,41 +733,52 @@ remove_by_name <- function(x, name) {
 
   ## Drop internal variable, unless place holder
   var <- map[idx]
-  if (!is.na(var)) remove(list=var, envir=x, inherits=FALSE)
+  if (!is.na(var)) remove(list = var, envir = x, inherits = FALSE)
 
   map <- map[-idx]
-  map(x) <- map
+  mapping(x) <- map
+
+  ## Remove dimensions
+  names <- names(x)
+  dim(x) <- NULL
+  names(x) <- names
 
   invisible(x)
-} # remove_by_name()
+}
 
 
 remove_by_index <- function(x, i) {
   ## Argument 'i':
   if (length(i) == 0L) {
-    stop("Cannot remove element. Zero-length index.", call.=FALSE)
+    stop("Cannot remove element. Zero-length index.", call. = FALSE)
   } else if (length(i) > 1L) {
-    stop("Cannot remove element. More than one index specified: ", hpaste(i), call.=FALSE)
+    stop("Cannot remove element. More than one index specified: ", hpaste(i),
+         call. = FALSE)
   } else if (!is.finite(i)) {
-    stop("Cannot remove element. Non-finite index: ", i, call.=FALSE)
+    stop("Cannot remove element. Non-finite index: ", i, call. = FALSE)
   } else if (i < 1L) {
-    stop("Cannot remove element. Non-positive index: ", i, call.=FALSE)
+    stop("Cannot remove element. Non-positive index: ", i, call. = FALSE)
   }
 
-  map <- map(x)
+  map <- mapping(x)
 
   ## Nothing to do?
   if (i > length(map)) return(invisible(x))
 
   ## Drop internal variable, unless place holder
   var <- map[i]
-  if (!is.na(var)) remove(list=var, envir=x, inherits=FALSE)
+  if (!is.na(var)) remove(list = var, envir = x, inherits = FALSE)
 
   map <- map[-i]
-  map(x) <- map
+  mapping(x) <- map
+
+  ## Remove dimensions
+  names <- names(x)
+  dim(x) <- NULL
+  names(x) <- names
 
   invisible(x)
-} # remove_by_index()
+}
 
 
 
@@ -737,30 +795,29 @@ remove_by_index <- function(x, i) {
 #' @keywords internal
 `$<-.listenv` <- function(x, name, value) {
   if (is.null(value)) {
-    remove_by_name(x, name=name)
+    remove_by_name(x, name = name)
   } else {
-    assign_by_name(x, name=name, value=value)
+    assign_by_name(x, name = name, value = value)
   }
 }
 
 #' @export
 `[[<-.listenv` <- function(x, ..., value) {
-  map <- map(x)
-  n <- length(map)
+  map <- mapping(x)
 
   idxs <- list(...)
   nidxs <- length(idxs)
 
   ## Subsetting by multiple dimensions?
   if (nidxs > 1L) {
-    i <- toIndex(x, idxs)
+    i <- to_index(x, idxs)
   } else {
     i <- idxs[[1L]]
     if (is.character(i)) {
       if (is.null(value)) {
-        x <- remove_by_name(x, name=i)
+        x <- remove_by_name(x, name = i)
       } else {
-        x <- assign_by_name(x, name=i, value=value)
+        x <- assign_by_name(x, name = i, value = value)
       }
       return(invisible(x))
     }
@@ -768,12 +825,12 @@ remove_by_index <- function(x, i) {
 
   if (is.numeric(i)) {
     if (is.null(value)) {
-      x <- remove_by_index(x, i=i)
+      x <- remove_by_index(x, i = i)
     } else {
-      x <- assign_by_index(x, i=i, value=value)
+      x <- assign_by_index(x, i = i, value = value)
     }
   } else {
-    stop(sprintf("Subsetted [[<- assignment to listenv's is only supported for names and indices, not %s", mode(i)), call.=FALSE)
+    stopf("Subsetted [[<- assignment to listenv's is only supported for names and indices, not %s", mode(i), call. = FALSE)  #nolint
   }
 
   return(invisible(x))
@@ -783,7 +840,7 @@ remove_by_index <- function(x, i) {
 #' @export
 `[<-.listenv` <- function(x, ..., value) {
   ## Need to allow for implicit indices, e.g. x[1,,2]
-  idxs <- as.list(sys.call())[-(1:2)]
+  idxs <- as.list(sys.call())[-(1:2)]  #nolint
   idxs$value <- NULL
   nidxs <- length(idxs)
 
@@ -791,11 +848,50 @@ remove_by_index <- function(x, i) {
   dim <- dim(x)
   ndim <- length(dim)
   if (nidxs > 1 && nidxs != ndim) {
-    stop(sprintf("Incorrect subsetting. Expected %d dimensions but got %d", ndim, nidxs))
+    stopf("Incorrect subsetting. Expected %d dimensions but got %d",
+          ndim, nidxs)
   }
 
   ## Implicitly specified dimensions
-  missing <- sapply(idxs, FUN=function(x) is.symbol(x) && identical("", deparse(x)))
+  missing <- sapply(idxs, FUN = function(x) {
+    is.symbol(x) && identical("", deparse(x))
+  })
+
+  ## Drop elements from matrix or array, e.g. x[,2] <- NULL?
+  if (ndim > 0L && nidxs > 1L && is.null(value)) {
+    if (ndim - sum(missing) != 1L) {
+      stop("Only one dimension at the time can be dropped when assigning NULL")
+    }
+    envir <- parent.frame()
+    dimnames <- dimnames(x)
+    array_idxs <- array(seq_along(x), dim = dim)
+    dimnames(array_idxs) <- dimnames
+    args <- c(list(array_idxs), idxs, list(drop = FALSE))
+    idxs_drop <- do.call(`[`, args = args)
+    for (dd in which(!missing)) {
+      idxs_dd <- idxs[[dd]]
+      idxs_dd <- eval(idxs_dd, envir = envir, enclos = baseenv())
+      if (length(idxs_dd) == 0) next
+      if (is.logical(idxs_dd)) {
+        idxs_dd <- rep(idxs_dd, length.out = dim[dd])
+        idxs_dd <- which(idxs_dd)
+      } else if (is.character(idxs_dd)) {
+        idxs_dd <- unique(idxs_dd)
+        idxs_dd <- match(idxs_dd, dimnames[[dd]])
+      } else {
+        idxs_dd <- unique(idxs_dd)
+      }
+      stop_if_not(is.numeric(idxs_dd))
+      dim[dd] <- dim[dd] - length(idxs_dd)
+      dimnames[[dd]] <- dimnames[[dd]][-idxs_dd]
+    }
+    idxs_drop <- sort(unique(idxs_drop), decreasing = TRUE)
+    for (i in idxs_drop) x <- remove_by_index(x, i = i)
+    dim(x) <- dim
+    dimnames(x) <- dimnames
+    return(invisible(x))
+  }
+  
   if (any(missing)) {
     if (nidxs == ndim) {
       envir <- parent.frame()
@@ -803,7 +899,7 @@ remove_by_index <- function(x, i) {
         if (missing[kk]) {
           idxs[[kk]] <- seq_len(dim[kk])
         } else {
-          idxs[[kk]] <- eval(idxs[[kk]], envir=envir)
+          idxs[[kk]] <- eval(idxs[[kk]], envir = envir, enclos = baseenv())
         }
       }
     } else if (nidxs == 1) {
@@ -811,25 +907,25 @@ remove_by_index <- function(x, i) {
         idxs <- list(seq_len(length(x)))
       } else {
         ## Special case: Preserve dimensions when x[]
-        idxs <- lapply(dim, FUN=function(n) seq_len(n))
+        idxs <- lapply(dim, FUN = function(n) seq_len(n))
         nidxs <- length(idxs)
      }
     }
   } else {
     envir <- parent.frame()
-    idxs <- lapply(idxs, FUN=eval, envir=envir)
+    idxs <- lapply(idxs, FUN = eval, envir = envir, enclos = baseenv())
   }
 
   if (nidxs <= 1L) {
     i <- idxs[[1L]]
   } else {
-    i <- toIndex(x, idxs)
+    i <- to_index(x, idxs)
   }
 
   ni <- length(i)
   if (is.logical(i)) {
     n <- length(x)
-    if (ni < n) i <- rep(i, length.out=n)
+    if (ni < n) i <- rep(i, length.out = n)
     i <- which(i)
     ni <- length(i)
   }
@@ -838,27 +934,41 @@ remove_by_index <- function(x, i) {
   # Nothing to do?
   if (ni == 0L) return(invisible(x))
 
+  if (!is.character(i) && !is.numeric(i)) {
+    stopf("Subsetted [<- assignment to listenv's is only supported for names and indices, not %s", mode(i), call. = FALSE)  #nolint
+  }
+  
+  # Remove elements?
+  if (is.null(value)) {
+    idxs <- unique(i)
+    if (is.character(i)) {
+      for (i in idxs) x <- remove_by_name(x, name = i)
+    } else if (is.numeric(i)) {
+      idxs <- sort(idxs, decreasing = TRUE)
+      for (i in idxs) x <- remove_by_index(x, i = i)
+    }
+    return(invisible(x))
+  }
+  
   nvalue <- length(value)
-  if (nvalue == 0L) stop("Replacement has zero length", call.=FALSE)
+  if (nvalue == 0L) stop("Replacement has zero length", call. = FALSE)
 
   if (ni != nvalue) {
     if (ni < nvalue || ni %% nvalue != 0) {
-      warning(sprintf("Number of items to replace is not a multiple of replacement length: %d != %d", ni, nvalue), call.=FALSE)
+      warnf("Number of items to replace is not a multiple of replacement length: %d != %d", ni, nvalue, call. = FALSE)  #nolint
     }
-    value <- rep(value, length.out=ni)
+    value <- rep(value, length.out = ni)
     nvalue <- length(value)
   }
 
   if (is.character(i)) {
     for (kk in seq_len(ni)) {
-      x <- assign_by_name(x, name=i[kk], value=value[[kk]])
+      x <- assign_by_name(x, name = i[kk], value = value[[kk]])
     }
   } else if (is.numeric(i)) {
     for (kk in seq_len(ni)) {
-      x <- assign_by_index(x, i=i[kk], value=value[[kk]])
+      x <- assign_by_index(x, i = i[kk], value = value[[kk]])
     }
-  } else {
-    stop(sprintf("Subsetted [<- assignment to listenv's is only supported for names and indices, not %s", mode(i)), call.=FALSE)
   }
   return(invisible(x))
 }
@@ -866,82 +976,38 @@ remove_by_index <- function(x, i) {
 
 #' @export
 #' @method unlist listenv
-unlist.listenv <- function(x, recursive=TRUE, use.names=TRUE) {
+unlist.listenv <- function(x, recursive = TRUE, use.names = TRUE) {  #nolint
   names <- names(x)
   x <- as.list(x)
   names(x) <- names
 
   if (recursive) {
     repeat {
-      x <- unlist(x, recursive=TRUE, use.names=use.names)
-      idxs <- unlist(lapply(x, FUN=inherits, "listenv"), use.names=FALSE)
+      x <- unlist(x, recursive = TRUE, use.names = use.names)
+      idxs <- unlist(lapply(x, FUN = inherits, "listenv"), use.names = FALSE)
       if (length(idxs) == 0L) break
       idxs <- which(idxs)
       if (length(idxs) == 0L) break
       for (ii in idxs) {
-        x[[ii]] <- unlist(x[[ii]], recursive=TRUE, use.names=use.names)
+        x[[ii]] <- unlist(x[[ii]], recursive = TRUE, use.names = use.names)
       }
     }
     x
   } else {
-    unlist(x, recursive=FALSE, use.names=use.names)
+    unlist(x, recursive = FALSE, use.names = use.names)
   }
 }
 
-#' @export
-dim.listenv <- function(x) attr(x, "dim.")
-
-#' @export
-`dim<-.listenv` <- function(x, value) {
-  n <- length(x)
-  if (!is.null(value)) {
-    names <- names(value)
-    value <- as.integer(value)
-    p <- prod(as.double(value))
-    if (p != n) {
-      stop(sprintf("dims [product %d] do not match the length of object [%d]", p, n))
-    }
-    names(value) <- names
-  }
-
-  ## Always remove "dimnames" and "names" attributes, cf. help("dim")
-  dimnames(x) <- NULL
-  names(x) <- NULL
-
-  attr(x, "dim.") <- value
-  x
-}
-
-
-#' @export
-dimnames.listenv <- function(x) attr(x, "dimnames.")
-
-#' @export
-`dimnames<-.listenv` <- function(x, value) {
-  dim <- dim(x)
-  if (is.null(dim) && !is.null(value)) {
-    stop("'dimnames' applied to non-array")
-  }
-  for (kk in seq_along(dim)) {
-    names <- value[[kk]]
-    if (is.null(names)) next
-    n <- length(names)
-    if (n != dim[kk]) {
-      stop(sprintf("length of 'dimnames' [%d] not equal to array extent", kk))
-    }
-  }
-  attr(x, "dimnames.") <- value
-  x
-}
 
 #' @export
 #' @method all.equal listenv
-all.equal.listenv <- function(target, current, all.names=TRUE, sorted=FALSE, ...) {
+all.equal.listenv <- function(target, current, all.names = TRUE,  #nolint
+                              sorted = FALSE, ...) {
   if (identical(target, current)) return(TRUE)
 
   ## Coerce to lists
-  target <- as.list(target, all.names=all.names, sorted=sorted)
-  current <- as.list(current, all.names=all.names, sorted=sorted)
+  target <- as.list(target, all.names = all.names, sorted = sorted)
+  current <- as.list(current, all.names = all.names, sorted = sorted)
 
   ## Not all as.list() methods support 'all.names'
   if (!all.names) {
@@ -956,5 +1022,5 @@ all.equal.listenv <- function(target, current, all.names=TRUE, sorted=FALSE, ...
     current <- current[order(names(current))]
   }
 
-  all.equal(target=target, current=current, ...)
+  all.equal(target = target, current = current, ...)
 }
